<?php

namespace Controller;

use Util\Messages;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Service\ContractedProductLine as ProductLineService;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class ContractedProductLine
{
    /** @var \Service\ContractedProductLine $productLineService */
    protected $productLineService;
    
    public function __construct(ProductLineService $productLineService)
    {
        $this->productLineService = $productLineService;
    }
    
    public function findContractedProductLine(Request\Filter $request, Response $response)
    {
        try{
            $filters               = $request->getFilterCriteria()->getConditions();
            $billingType           = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'BILLING_TYPE');
            $billingMonth          = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'BILLING_REFERRING_TO_MONTH');
            $contractedProductLine = $this->productLineService->findContractedProductLine($billingType, $billingMonth);
            $response->addDataSet(new Dataset('findContractedProductLine', $contractedProductLine));
        } catch(\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
            
    }

    public function verifyProductDiscount(Request\Filter $request, Response $response)
    {
        try{
            $doesItHaveDiscount = $this->productLineService->verifyProductDiscount();
            $response->addDataSet(new Dataset('doesItHaveDiscount', $doesItHaveDiscount));
        } catch(\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }

    }
    
    public function findBillingReferringToMonth(Request\Filter $request, Response $response)
    {
        try{
            $billingReferringToMonth = $this->productLineService->findBillingReferringToMonth();
            $response->addDataSet(new DataSet('findBillingReferringToMonth', $billingReferringToMonth));
        } catch(\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }
    
    public function findSystems(Request\Filter $request, Response $response)
    {
        try{
            $filters        = $request->getFilterCriteria()->getConditions();
            $productLineId  = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'PRODUCT_LINE_ID', true);
            $billingType    = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'BILLING_TYPE', true);
            $billingMonth   = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'BILLING_MONTH', true);
            $conditions     = DevOpsUtilities::createQueryFiltered($filters);
            $systems        = $this->productLineService->findSystems($productLineId, $billingType, $billingMonth, $conditions);
            $response->addDataSet(new DataSet('findSystems', $systems));
        } catch(\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }
    
    public function findUseLicenses(Request\Filter $request, Response $response)
    {
        try{
            $filters        = $request->getFilterCriteria()->getConditions();
            $productLineId  = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'PRODUCT_LINE_ID', true);
            $billingType    = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'BILLING_TYPE', true);
            $billingMonth   = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'BILLING_MONTH', true);
            $controlForm    = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'CONTROL_FORM', true);
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            $useLicenses    = $this->productLineService->findUseLicenses($productLineId, $billingType, $billingMonth,
                $controlForm, $filters, $orderBy);
            $response->addDataSet(new DataSet('findUseLicenses', $useLicenses));
        } catch(\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }
    
    
}